--[[
	Weapon: Ventilator
	by: Kiffer-Opa
	version 1.0
]]
-- Setup Tables
if kifferopa==nil then kifferopa={} end
kifferopa.ventilator={}
kifferopa.ventilator.ventilator={}
kifferopa.ventilator.settings={}

-- SETTINGS --
--[[ power:
     how much wind one ventilator changes. Minimum can be as small if you wish (if it still makes sense to you)
     and a maximum value that makes sense is 0.2. This value and all values above it will force the wind direction
     to the total max left/right. This is because wind ranges from -0.1 to 0.1. So even if natural wind if max power
     left, ventilator forces the wind to go to the max right.
     note that two ventilators facing in opposite directions together don't change the wind (neutralization)
     and multiple ventilators that face in the same direction cause together multiple wind, but it does of course
     never get stronger than the wind limit.]]
kifferopa.ventilator.settings.power=0.1
--[[ animationspeed:
     how fast the fan of the ventilator turns. To be exact, how many time frames (1 second=50 frames) to be waited
     before the fan shows the next animation image. Minimum value is 1, this will be the FASTEST animation speed.
     Higher values mean slower animation.
     Only whole numbers are allowed.
]]
kifferopa.ventilator.settings.animationspeed=1
-- END OF SETTINGS --

-- Load & Prepare Ressources
kifferopa.ventilator.sfx_place=loadsfx("throw.ogg")							-- Attack Sound
kifferopa.ventilator.sfx_bounce=loadsfx("bounce.wav")							-- Bounce
kifferopa.ventilator.gfx_ventilator={}									-- ventilator
for i=0,3 do
	kifferopa.ventilator.gfx_ventilator[i]=loadgfx("buildings/ventilator"..i..".png")
	setmidhandle(kifferopa.ventilator.gfx_ventilator[i])
end
kifferopa.ventilator.gfx_ventilator.mirrored=loadgfx("buildings/ventilator0_mirrored.png")
setmidhandle(kifferopa.ventilator.gfx_ventilator.mirrored)
kifferopa.ventilator.gfx_icon=loadgfx("buildings/ventilator_icon.png")
setmidhandle(kifferopa.ventilator.gfx_icon)

--[[
	Weapon: Ventilator
]]

kifferopa.ventilator.id=addweapon("kifferopa.ventilator","Ventilator",kifferopa.ventilator.gfx_icon,0,2)

function kifferopa.ventilator.draw()									-- Draw
	if(weapon_shots==0) then
		local img
		if(getplayerdirection(0)==1) then
			img=kifferopa.ventilator.gfx_ventilator[0]
		else
			img=kifferopa.ventilator.gfx_ventilator.mirrored
		end
		hudpositioning(pos_build,img,30,1,1,1)
		hudinfo("The ventilator will face to the same direction as you do.")
	end
end

function kifferopa.ventilator.attack(attack)												-- Attack
	if (weapon_shots<=0) and (weapon_position==1) then
		-- No more weapon switching!
		useweapon(0)
		playsound(kifferopa.ventilator.sfx_place)
		weapon_shots=weapon_shots+1
		createobject(kifferopa.ventilator.ventilator.id,weapon_x,weapon_y)
		-- End Turn
		endturn()
	end
end

--[[
	Object: Ventilator
]]

kifferopa.ventilator.ventilator.id=addobject("kifferopa.ventilator.ventilator",kifferopa.ventilator.gfx_ventilator[1])

function kifferopa.ventilator.ventilator.setup(id,parameter)
	if objects==nil then objects={} end
	objects[id]={}
	objects[id].sy=0
	objects[id].timer=0
	objects[id].health=30
	objects[id].dir=getplayerdirection(0)
	objects[id].frame=1
end

function kifferopa.ventilator.ventilator.draw(id,x,y)
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(objects[id].dir,1)
	setrotation(0)
	-- Draw ventilator 
	if(getframe()%kifferopa.ventilator.settings.animationspeed==0) then
		objects[id].frame=(objects[id].frame-1)%4
	end
	drawimage(kifferopa.ventilator.gfx_ventilator[objects[id].frame],x,y)
end

function kifferopa.ventilator.ventilator.update(id,x,y)
	-- change wind on turn start
	if(getframe()==0) then
		local ventilators=objecttable(kifferopa.ventilator.ventilator.id)
		local smallest=nil
		for i=1,#ventilators do
			o=ventilators[i]
			if(smallest==nil) then
				smallest=o
			elseif(o<smallest) then
				smallest=o
			end
		end
		--[[ if this ventilator has smallest ID, it changes the wind. Yeah, that's some dirty trick
		     but it works.]]
		if(id==smallest) then
			kifferopa.ventilator.updatewind()
		end
	end

	-- Gravity influence on speed
	objects[id].sy=objects[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.abs(objects[id].sy)/15)
	msuby=objects[id].sy/msubt
	local collided=false
	for i=1,msubt,1 do
		y=y+msuby
		-- Collision
		if collision(kifferopa.ventilator.gfx_ventilator[1],x,y,1,1,1,-1,id)==1 then
			if (math.abs(objects[id].sy)>0.5) then playsound(kifferopa.ventilator.sfx_bounce) end
			y=y-msuby
			objects[id].sy=-objects[id].sy*0.3
			msuby=-msuby*0.3
			collided=true
		end
		-- Water
		if y>getwatery()+5 then
			-- Effects
			particle(p_waterhit,x,y)
			playsound(sfx_hitwater1)
			-- Remove
			removeobject(id)
			return
		end
	end
	objectposition(id,x,y)

--[[
	if(math.abs(objects[id].sy)>0.01 and collided==true) then
		return 1
	end
]]
end

function kifferopa.ventilator.ventilator.damage(id,damage)
	if objects[id].health>0 then
		objects[id].health=objects[id].health-damage
		if objects[id].health<=0 then
			local x=getobjectx(id)
			local y=getobjecty(id)
			-- Destroy terrain
			terrainexplosion(x,y,35,1)
			-- Crater
			grey=math.random(0,40)
			if math.random(0,1)==1 then
				terrainalphaimage(gfx_crater100,x,y,math.random(6,9)*0.1,grey,grey,grey)
			else
				terrainalphaimage(gfx_crater125,x,y,math.random(6,9)*0.1,grey,grey,grey)
			end
			-- Remove
			removeobject(id)
			-- Cause damage
			arealdamage(x,y,65,35)
		end
	end
end

function kifferopa.ventilator.updatewind()
	local ventilators=objecttable(kifferopa.ventilator.ventilator.id)
	local windchange=0
	--[[ get the total wind power (summed up of all ventilators on map
	     and check wheather this ventilator has the smallest ID. ]]
	for i=1,#ventilators do
		o=ventilators[i]
		windchange=windchange+objects[o].dir*kifferopa.ventilator.settings.power
	end
	wind(getwind()+windchange)
end
